"use strict";
// Copyright (c) Martin Renou
// Distributed under the terms of the Modified BSD License.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SceneView = exports.SceneModel = exports.WaterModel = exports.UnderWaterModel = exports.ThresholdModel = exports.IsoSurfaceModel = exports.ColorBarView = exports.ColorBarModel = exports.IsoColorModel = exports.RGBModel = exports.AlphaModel = exports.WarpByScalarModel = exports.WarpModel = exports.PointCloudModel = exports.TetraMeshModel = exports.PolyMeshModel = exports.DataModel = exports.ComponentModel = void 0;
const THREE = __importStar(require("three"));
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
// Import the CSS
require("../css/gany.css");
const ganyjs_1 = require("ganyjs");
const colormaps = [
    'BrBG', 'PRGn', 'PiYG', 'PuOr', 'RdBu', 'RdGy', 'RdYlBu', 'RdYlGn',
    'Spectral', 'BuGn', 'BuPu', 'GnBu', 'OrRd', 'PuBuGn', 'PuBu', 'PuRd', 'RdPu',
    'YlGnBu', 'YlGn', 'YlOrBr', 'YlOrRd', 'Blues', 'Greens', 'Greys', 'Purples', 'Reds',
    'Oranges', 'Cividis', 'CubehelixDefault', 'Rainbow', 'Warm', 'Cool', 'Sinebow', 'Turbo',
    'Viridis', 'Magma', 'Inferno', 'Plasma'
];
function deserialize_float32array(data, manager) {
    return new Float32Array(data.data.buffer);
}
function deserialize_uint32array(data, manager) {
    return new Uint32Array(data.data.buffer);
}
function deserialize_data_array(value, manager) {
    if (typeof value == 'string') {
        return base_1.unpack_models(value, manager);
    }
    else {
        return deserialize_float32array(value, manager);
    }
}
class _GanyWidgetModel extends base_1.WidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_module: _GanyWidgetModel.model_module, _model_module_version: _GanyWidgetModel.model_module_version });
    }
}
_GanyWidgetModel.model_module = version_1.MODULE_NAME;
_GanyWidgetModel.model_module_version = version_1.MODULE_VERSION;
class _GanyDOMWidgetModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_module: _GanyDOMWidgetModel.model_module, _model_module_version: _GanyDOMWidgetModel.model_module_version, _view_module: _GanyDOMWidgetModel.view_module, _view_module_version: _GanyDOMWidgetModel.view_module_version });
    }
}
_GanyDOMWidgetModel.model_module = version_1.MODULE_NAME;
_GanyDOMWidgetModel.model_module_version = version_1.MODULE_VERSION;
_GanyDOMWidgetModel.view_module = version_1.MODULE_NAME;
_GanyDOMWidgetModel.view_module_version = version_1.MODULE_VERSION;
class ComponentModel extends _GanyWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ComponentModel.model_name, name: '', array: [] });
    }
    get array() {
        const array = this.get('array');
        if (array.hasOwnProperty('name') && array.name == 'NDArrayModel') {
            return new Float32Array(array.getNDArray().data);
        }
        else {
            return array;
        }
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.component = new ganyjs_1.Component(this.get('name'), this.array);
        this.on('change:array', () => { this.component.array = this.array; });
    }
}
exports.ComponentModel = ComponentModel;
ComponentModel.serializers = Object.assign(Object.assign({}, _GanyWidgetModel.serializers), { array: { deserialize: deserialize_data_array } });
ComponentModel.model_name = 'ComponentModel';
class DataModel extends _GanyWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: DataModel.model_name, name: '', components: [] });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        const components = this.get('components').map((componentModel) => {
            return componentModel.component;
        });
        this.data = new ganyjs_1.Data(this.get('name'), components);
    }
}
exports.DataModel = DataModel;
DataModel.serializers = Object.assign(Object.assign({}, _GanyWidgetModel.serializers), { components: { deserialize: base_1.unpack_models } });
DataModel.model_name = 'DataModel';
class BlockModel extends _GanyWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: BlockModel.model_name, vertices: [], data: [], environment_meshes: [], default_color: '#6395b0' });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.block = this.createBlock();
        this.block.defaultColor = this.defaultColor;
        this.initEventListeners();
    }
    get vertices() {
        const array = this.get('vertices');
        if (array.hasOwnProperty('name') && array.name == 'NDArrayModel') {
            return new Float32Array(array.getNDArray().data);
        }
        else {
            return array;
        }
    }
    get data() {
        return this.get('data').map((dataModel) => dataModel.data);
        ;
    }
    get environmentMeshes() {
        return this.get('environment_meshes').map((model) => model.obj);
    }
    get defaultColor() {
        return new THREE.Color(this.get('default_color'));
    }
    initEventListeners() {
        this.on('change:default_color', () => { this.block.defaultColor = this.defaultColor; });
    }
}
BlockModel.serializers = Object.assign(Object.assign({}, _GanyWidgetModel.serializers), { vertices: { deserialize: deserialize_data_array }, data: { deserialize: base_1.unpack_models }, environment_meshes: { deserialize: base_1.unpack_models } });
BlockModel.model_name = 'BlockModel';
class PolyMeshModel extends BlockModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PolyMeshModel.model_name, triangle_indices: [] });
    }
    createBlock() {
        return new ganyjs_1.PolyMesh(this.vertices, this.triangleIndices, this.data, { environmentMeshes: this.environmentMeshes });
    }
    get triangleIndices() {
        return this.get('triangle_indices');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:vertices', () => { this.block.vertices = this.vertices; });
        this.on('change:triangle_indices', () => { this.block.triangleIndices = this.triangleIndices; });
    }
}
exports.PolyMeshModel = PolyMeshModel;
PolyMeshModel.serializers = Object.assign(Object.assign({}, BlockModel.serializers), { triangle_indices: { deserialize: deserialize_uint32array } });
PolyMeshModel.model_name = 'PolyMeshModel';
class TetraMeshModel extends PolyMeshModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: TetraMeshModel.model_name, tetrahedron_indices: [] });
    }
    createBlock() {
        return new ganyjs_1.TetraMesh(this.vertices, this.triangleIndices, this.tetrahedronIndices, this.data, { environmentMeshes: this.environmentMeshes });
    }
    get tetrahedronIndices() {
        return this.get('tetrahedron_indices');
    }
}
exports.TetraMeshModel = TetraMeshModel;
TetraMeshModel.serializers = Object.assign(Object.assign({}, PolyMeshModel.serializers), { tetrahedron_indices: { deserialize: deserialize_uint32array } });
TetraMeshModel.model_name = 'TetraMeshModel';
class PointCloudModel extends BlockModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PointCloudModel.model_name });
    }
    createBlock() {
        return new ganyjs_1.PointCloud(this.vertices, this.data, { environmentMeshes: this.environmentMeshes });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:vertices', () => { this.block.vertices = this.vertices; });
    }
}
exports.PointCloudModel = PointCloudModel;
PointCloudModel.model_name = 'PointCloudModel';
class EffectModel extends BlockModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: EffectModel.model_name, parent: null, input: null });
    }
    get parent() {
        return this.get('parent');
    }
    get input() {
        return this.get('input');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:input', this.updateInput.bind(this));
    }
    updateInput() {
        if (this.block.inputDimension != 0) {
            this.block.setInput(this.input);
        }
    }
}
EffectModel.serializers = Object.assign(Object.assign({}, BlockModel.serializers), { parent: { deserialize: base_1.unpack_models } });
EffectModel.model_name = 'EffectModel';
class WarpModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: WarpModel.model_name });
    }
    get input() {
        return this.get('input');
    }
    get offset() {
        const offset = this.get('offset');
        if (typeof offset == 'number') {
            return new THREE.Vector3(offset, offset, offset);
        }
        else {
            return new THREE.Vector3(offset[0], offset[1], offset[2]);
        }
    }
    get factor() {
        const factor = this.get('factor');
        if (typeof factor == 'number') {
            return new THREE.Vector3(factor, factor, factor);
        }
        else {
            return new THREE.Vector3(factor[0], factor[1], factor[2]);
            ;
        }
    }
    createBlock() {
        return new ganyjs_1.Warp(this.parent.block, this.input, this.factor, this.offset);
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:factor', () => { this.block.factor = this.factor; });
        this.on('change:offset', () => { this.block.offset = this.offset; });
    }
}
exports.WarpModel = WarpModel;
WarpModel.model_name = 'WarpModel';
class WarpByScalarModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: WarpByScalarModel.model_name });
    }
    get input() {
        return this.get('input');
    }
    get factor() {
        return this.get('factor');
    }
    createBlock() {
        return new ganyjs_1.WarpByScalar(this.parent.block, this.input, this.factor);
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:factor', () => { this.block.factor = this.factor; });
    }
}
exports.WarpByScalarModel = WarpByScalarModel;
WarpByScalarModel.model_name = 'WarpByScalarModel';
class AlphaModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AlphaModel.model_name });
    }
    get input() {
        const input = this.get('input');
        return typeof input == 'string' ? input : [input];
    }
    createBlock() {
        return new ganyjs_1.Alpha(this.parent.block, this.input);
    }
}
exports.AlphaModel = AlphaModel;
AlphaModel.model_name = 'AlphaModel';
class RGBModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: RGBModel.model_name });
    }
    get input() {
        return this.get('input');
    }
    createBlock() {
        return new ganyjs_1.RGB(this.parent.block, this.input);
    }
}
exports.RGBModel = RGBModel;
RGBModel.model_name = 'RGBModel';
class IsoColorModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: IsoColorModel.model_name, min: 0., max: 0., range: [0., 0.], colormap: 34, type: 'linear' });
    }
    get min() {
        return this.get('min');
    }
    set min(value) {
        this.set('min', value);
    }
    get max() {
        return this.get('max');
    }
    set max(value) {
        this.set('max', value);
    }
    get range() {
        return this.get('range');
    }
    set range(value) {
        this.set('range', value);
    }
    get colormap() {
        return colormaps[this.get('colormap')];
    }
    get type() {
        return this.get('type');
    }
    get input() {
        const input = this.get('input');
        return typeof input == 'string' ? input : [input];
    }
    createBlock() {
        return new ganyjs_1.IsoColor(this.parent.block, this.input, this.min, this.max, this.colormap, this.type);
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:min', () => {
            this.block.min = this.min;
            this.range = [this.min, this.range[1]];
        });
        this.on('change:max', () => {
            this.block.max = this.max;
            this.range = [this.range[0], this.max];
        });
        this.on('change:range', () => {
            this.min = this.range[0];
            this.max = this.range[1];
        });
        this.on('change:colormap', () => { this.block.colorMap = this.colormap; });
        this.on('change:type', () => { this.block.type = this.type; });
    }
}
exports.IsoColorModel = IsoColorModel;
IsoColorModel.model_name = 'IsoColorModel';
class ColorBarModel extends _GanyDOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ColorBarModel.model_name, _view_name: ColorBarModel.view_name, parent: null });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.colorBar = this.parent.block.colorBar;
        this.parent.block.on('change:colorbar', this.redraw.bind(this));
        this.redraw();
        this.on('change:parent', this.updateColorBar.bind(this));
    }
    get parent() {
        return this.get('parent');
    }
    updateColorBar() {
        // TODO Turn off events on old parent
        this.colorBar = this.parent.block.colorBar;
        this.parent.block.on('change:colorbar', this.redraw.bind(this));
        this.redraw();
    }
    redraw() {
        for (const view_id in this.views) {
            this.views[view_id].then((view) => {
                view.draw();
            });
        }
    }
}
exports.ColorBarModel = ColorBarModel;
ColorBarModel.serializers = Object.assign(Object.assign({}, _GanyDOMWidgetModel.serializers), { parent: { deserialize: base_1.unpack_models } });
ColorBarModel.model_name = 'ColorBarModel';
ColorBarModel.view_name = 'ColorBarView';
class ColorBarView extends base_1.DOMWidgetView {
    render() {
        const ctx = this.el.getContext('2d');
        this.el.classList.add('gany-colorbar');
        if (ctx === null) {
            throw 'Failed to create canvas context for ColorBarView';
        }
        this.ctx = ctx;
        this.draw();
    }
    draw() {
        this.el.width = this.model.colorBar.width;
        this.el.height = this.model.colorBar.height;
        this.ctx.drawImage(this.model.colorBar, 0, 0);
    }
    // @ts-ignore
    get tagName() {
        return 'canvas';
    }
}
exports.ColorBarView = ColorBarView;
class IsoSurfaceModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: IsoSurfaceModel.model_name, value: 0., dynamic: false });
    }
    get value() {
        return this.get('value');
    }
    get dynamic() {
        return this.get('dynamic');
    }
    get input() {
        const input = this.get('input');
        return typeof input == 'string' ? input : [input];
    }
    createBlock() {
        return new ganyjs_1.IsoSurface(this.parent.block, this.input, { value: this.value, dynamic: this.dynamic });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:value', () => { this.block.value = this.value; });
    }
}
exports.IsoSurfaceModel = IsoSurfaceModel;
IsoSurfaceModel.model_name = 'IsoSurfaceModel';
class ThresholdModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ThresholdModel.model_name, min: 0., max: 0., range: [0., 0.], dynamic: false, inclusive: true });
    }
    get min() {
        return this.get('min');
    }
    set min(value) {
        this.set('min', value);
    }
    get max() {
        return this.get('max');
    }
    set max(value) {
        this.set('max', value);
    }
    get range() {
        return this.get('range');
    }
    set range(value) {
        this.set('range', value);
    }
    get dynamic() {
        return this.get('dynamic');
    }
    get inclusive() {
        return this.get('inclusive');
    }
    get input() {
        const input = this.get('input');
        return typeof input == 'string' ? input : [input];
    }
    createBlock() {
        return new ganyjs_1.Threshold(this.parent.block, this.input, {
            min: this.min, max: this.max,
            dynamic: this.dynamic, inclusive: this.inclusive
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:min', () => {
            this.block.min = this.min;
            this.range = [this.min, this.range[1]];
        });
        this.on('change:max', () => {
            this.block.max = this.max;
            this.range = [this.range[0], this.max];
        });
        this.on('change:range', () => {
            this.min = this.range[0];
            this.max = this.range[1];
        });
        this.on('change:inclusive', () => { this.block.inclusive = this.inclusive; });
    }
}
exports.ThresholdModel = ThresholdModel;
ThresholdModel.model_name = 'ThresholdModel';
class UnderWaterModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: UnderWaterModel.model_name, default_color: '#F2FFD2', texture: null, texture_scale: 2., texture_position: [1., 1., 0.] });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.setTexture();
    }
    get input() {
        const input = this.get('input');
        return typeof input == 'string' ? input : [input];
    }
    get textureScale() {
        return this.get('texture_scale');
    }
    get texturePosition() {
        const texturePosition = this.get('texture_position');
        return new THREE.Vector3(texturePosition[0], texturePosition[1], texturePosition[2]);
    }
    setTexture() {
        const image = this.get('texture');
        if (image === null) {
            this.block.texture = null;
            return;
        }
        this.widget_manager.create_view(image, {}).then((imageView) => {
            const textureLoader = new THREE.TextureLoader();
            textureLoader.load(imageView.el.src, (texture) => {
                this.block.texture = texture;
            });
        });
    }
    createBlock() {
        return new ganyjs_1.UnderWater(this.parent.block, this.input, { defaultColor: this.defaultColor, textureScale: this.textureScale, texturePosition: this.texturePosition });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:texture', () => {
            this.setTexture();
        });
        this.on('change:texture_scale', () => {
            this.block.textureScale = this.textureScale;
        });
        this.on('change:texture_position', () => {
            this.block.texturePosition = this.texturePosition;
        });
    }
}
exports.UnderWaterModel = UnderWaterModel;
UnderWaterModel.model_name = 'UnderWaterModel';
UnderWaterModel.serializers = Object.assign(Object.assign({}, EffectModel.serializers), { texture: { deserialize: base_1.unpack_models } });
class WaterModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: WaterModel.model_name, under_water_blocks: [], caustics_enabled: false, caustics_factor: 0.2 });
    }
    get causticsFactor() {
        return this.get('caustics_factor');
    }
    get causticsEnabled() {
        return this.get('caustics_enabled');
    }
    get underWaterBlocks() {
        return this.get('under_water_blocks').map((underWaterBlockWidget) => underWaterBlockWidget.block);
    }
    createBlock() {
        // TODO Add texture to the package
        const url = 'https://raw.githubusercontent.com/martinRenou/threejs-caustics/master/assets/TropicalSunnyDay_';
        return new ganyjs_1.Water(this.parent.block, {
            underWaterBlocks: this.underWaterBlocks,
            causticsEnabled: this.causticsEnabled,
            causticsFactor: this.causticsFactor,
            skybox: new THREE.CubeTextureLoader().load([
                url + 'px.jpg', url + 'nx.jpg',
                url + 'py.jpg', url + 'ny.jpg',
                url + 'pz.jpg', url + 'nz.jpg',
            ]),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:caustics_enabled', () => { this.block.causticsEnabled = this.causticsEnabled; });
        this.on('change:caustics_factor', () => { this.block.causticsFactor = this.causticsFactor; });
    }
}
exports.WaterModel = WaterModel;
WaterModel.model_name = 'WaterModel';
WaterModel.serializers = Object.assign(Object.assign({}, EffectModel.serializers), { under_water_blocks: { deserialize: base_1.unpack_models } });
class SceneModel extends _GanyDOMWidgetModel {
    constructor() {
        super(...arguments);
        this.cameraNear = 0.01;
        this.cameraFar = 99999;
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SceneModel.model_name, _view_name: SceneModel.view_name, background_color: 'white', background_opacity: 1., children: [], camera: null });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.scene = new ganyjs_1.Scene();
        this.updateChildren();
        this.on('change:children', this.updateChildren.bind(this));
        this.on('change:camera', this.updateCamera.bind(this));
    }
    get backgroundColor() {
        return this.get('background_color');
    }
    get backgroundOpacity() {
        return this.get('background_opacity');
    }
    get camera() {
        return this.get('camera');
    }
    set camera(value) {
        this.set('camera', value);
    }
    updateCamera() {
        const blocks = this.get('children').map((child) => child.block);
        if (blocks.length == 0) {
            return;
        }
        // TODO Compute mean radius and mean center of all the blocks
        const radius = blocks[0].boundingSphere.radius;
        const center = blocks[0].boundingSphere.center;
        const modelCameraPosition = this.camera ? this.camera['position'] || null : null;
        const modelCameraRotation = this.camera ? this.camera['rotation'] || null : null;
        const modelCameraTarget = this.camera ? this.camera['target'] || null : null;
        const modelCameraUp = this.camera ? this.camera['up'] || null : null;
        if (modelCameraPosition) {
            this.cameraPosition = new THREE.Vector3(modelCameraPosition[0], modelCameraPosition[1], modelCameraPosition[2]);
        }
        else {
            this.cameraPosition = new THREE.Vector3(center.x, center.y, center.z + 2.5 * radius);
        }
        if (modelCameraRotation) {
            this.cameraRotation = new THREE.Quaternion(modelCameraRotation[0], modelCameraRotation[1], modelCameraRotation[2], modelCameraRotation[3]);
        }
        if (modelCameraTarget) {
            this.cameraTarget = new THREE.Vector3(modelCameraTarget[0], modelCameraTarget[1], modelCameraTarget[2]);
        }
        else {
            this.cameraTarget = new THREE.Vector3(center.x, center.y, center.z);
        }
        if (modelCameraUp) {
            this.cameraUp = new THREE.Vector3(modelCameraUp[0], modelCameraUp[1], modelCameraUp[2]);
        }
        this.cameraNear = radius / 3.;
        this.cameraFar = radius * 10.;
        this.trigger('update_camera');
    }
    updateChildren() {
        // TODO: Remove old children
        const blocks = this.get('children').map((child) => child.block);
        if (blocks.length == 0) {
            return;
        }
        for (const block of blocks) {
            this.scene.addBlock(block);
        }
        this.updateCamera();
    }
}
exports.SceneModel = SceneModel;
SceneModel.serializers = Object.assign(Object.assign({}, _GanyDOMWidgetModel.serializers), { children: { deserialize: base_1.unpack_models } });
SceneModel.model_name = 'SceneModel';
SceneModel.view_name = 'SceneView';
class SceneView extends base_1.DOMWidgetView {
    render() {
        this.el.classList.add('gany-scene');
        this.renderer = new ganyjs_1.Renderer(this.el, this.model.scene);
        this.displayed.then(() => {
            this.renderer.initialize();
            this.renderer.backgroundColor = this.model.backgroundColor;
            this.renderer.backgroundOpacity = this.model.backgroundOpacity;
            this.updateCamera();
            this.initEventListeners();
        });
    }
    initEventListeners() {
        window.addEventListener('resize', this.resize.bind(this), false);
        this.model.on('change:background_color', () => { this.renderer.backgroundColor = this.model.backgroundColor; });
        this.model.on('change:background_opacity', () => { this.renderer.backgroundOpacity = this.model.backgroundOpacity; });
        this.model.on('update_camera', this.updateCamera.bind(this));
        this.renderer.controls.addEventListener('change', this.handleCameraMove.bind(this));
    }
    updateCamera() {
        if (this.model.cameraPosition && !this.model.cameraPosition.equals(this.renderer.cameraPosition)) {
            this.renderer.cameraPosition = this.model.cameraPosition;
        }
        if (this.model.cameraRotation && !this.model.cameraRotation.equals(this.renderer.cameraRotation)) {
            this.renderer.cameraRotation = this.model.cameraRotation;
        }
        if (this.model.cameraTarget && !this.model.cameraTarget.equals(this.renderer.cameraTarget)) {
            this.renderer.cameraTarget = this.model.cameraTarget;
        }
        if (this.model.cameraUp && !this.model.cameraUp.equals(this.renderer.cameraUp)) {
            this.renderer.cameraUp = this.model.cameraUp;
        }
        this.renderer.camera.near = this.model.cameraNear;
        this.renderer.camera.far = this.model.cameraFar;
    }
    handleCameraMove() {
        this.model.camera = {
            position: this.renderer.cameraPosition.toArray(),
            rotation: this.renderer.cameraRotation.toArray(),
            target: this.renderer.cameraTarget.toArray(),
            up: this.renderer.cameraUp.toArray(),
        };
    }
    processPhosphorMessage(msg) {
        super.processPhosphorMessage(msg);
        switch (msg.type) {
            case 'resize':
            case 'after-show':
                this.resize();
                break;
        }
    }
    remove() {
        this.renderer.dispose();
        return super.remove();
    }
    resize() {
        this.renderer.resize();
    }
}
exports.SceneView = SceneView;
//# sourceMappingURL=widget.js.map